#!/bin/bash

# Checks for prerequisites: ant, a Tomcat installation, mysql
# Downloads and builds axis2 with rampart if it not already here
# updates the hostname in oscars.properties
# creates and populates the data bases with default values
# compiles the OSCARS services

echo "  ";
echo "--- Checking prerequisites...";
if [ -d src/net/es/oscars ]; then
    echo "    We seem to be in the correct directory";
else
    echo "";
    echo "Error. Run this script from its directory. ";
    exit 1;
fi

INSTALL_HOME=`pwd`;

# Find Ant
if [ `which ant` ]; then
    echo "    Found ant";
else
    echo "";
    echo "Ant not found. Install and add ant to your PATH and rerun installer.";
    exit 1;
fi

# Find Tomcat
if [ -n "$CATALINA_HOME" ]; then
    echo "    Environment variable CATALINA_HOME is set to $CATALINA_HOME";
else
    echo "";
    echo "Environment variable CATALINA_HOME is not set. Download and install Apache Tomcat v. 5.5 and set the CATALINA_HOME environment variable to the install root directory.";
    exit 1;
fi
echo "";
echo "";
echo "--- Your kit looks good.";

# Axis2 retrieve & build
bash tools/axis2/axis2_install.sh $INSTALL_HOME
if [ $? != 0 ]; then
    exit 1;
fi

POLICY_HOSTNAME=`hostname`;
echo -n "- Input the hostname for this Policy Service. Leave blank for \"$POLICY_HOSTNAME\": "
read READ_HOSTNAME;
if [ $READ_HOSTNAME ]; then
    POLICY_HOSTNAME="$READ_HOSTNAME";
fi
echo "    Using $POLICY_HOSTNAME . ";

READ_INSTALLDB=0;
while [ $READ_INSTALLDB == 0 ]; do
    echo -n "- Install databases y/n? "
    read READ_INSTALLDB;
    if [ "$READ_INSTALLDB" != "y" ] && [ "$READ_INSTALLDB" != "Y" ] && [ "$READ_INSTALLDB" != "n" ] && [ "$READ_INSTALLDB" != "N" ]; then
        READ_INSTALLDB=0;
    fi
done

INSTALL_DB=0;
if [ "$READ_INSTALLDB" == "y" ] || [ "$READ_INSTALLDB" == "Y" ]; then
    echo "    OK, will install databases.";
    INSTALL_DB=1;
fi

if [ $INSTALL_DB == 1 ]; then
    FOUND_MYSQL=0;
    if [ `which mysql` ]; then
        FOUND_MYSQL=`which mysql`;
        echo "    Found mysql client at $FOUND_MYSQL";
    else
        echo "    MySQL client not found. Add mysql to your PATH and rerun installer, or install databases manually.";
    fi


    if [ $FOUND_MYSQL != 0 ]; then
        MYSQL_PRIV_ACCESS=0;
        while [ $MYSQL_PRIV_ACCESS == 0 ]; do
            echo -n "- Input the MySQL server hostname. Leave blank for localhost: "
            read MYSQL_SERVER;
            if [ ! $MYSQL_SERVER ]; then
                MYSQL_SERVER="localhost";
            fi
             echo "    Using $MYSQL_SERVER . ";

            echo -n "- Input a privileged MySQL username on that host. Leave blank for root: "
            read MYSQL_PRIV_USERNAME;
            if [ ! "$MYSQL_PRIV_USERNAME" ]; then
                MYSQL_PRIV_USERNAME="root";
            fi
            echo "    Using $MYSQL_PRIV_USERNAME . ";

            echo -n "- Input the password for the privileged account: "
            stty -echo;
            read MYSQL_PRIV_PASSWORD;
            stty echo;
            echo "";

            `mysql --user=$MYSQL_PRIV_USERNAME --password="$MYSQL_PRIV_PASSWORD" --host=$MYSQL_SERVER --execute="STATUS" > /dev/null 2> /dev/null`;
            if [ $? != 0 ]; then
                echo "    Could not connect. Check password and try again. Ctrl-C to exit.";
            else
                echo "    Privileged account access verified.";
                MYSQL_PRIV_ACCESS=1;
            fi
            echo "";
        done

        MYSQL_GOT_POLICY_CREDENTIALS=0;

        while [ $MYSQL_GOT_POLICY_CREDENTIALS == 0 ]; do

            echo "- Input a MySQL username the Policy Service will use to connect to the databases.";
            echo "  -- This name and password must match the hibernate.connection.username and password specified in oscars.properties."
            echo -n "  Leave blank for \"oscars\": "
            read MYSQL_POLICY_USERNAME;
            if [ ! "$MYSQL_POLICY_USERNAME" ]; then
                MYSQL_POLICY_USERNAME="oscars";
            fi
            echo "    Using $MYSQL_POLICY_USERNAME . ";

            echo -n "- Input the password for the Policy Service account: "
            stty -echo;
            read MYSQL_POLICY_PASSWORD;
            stty echo;
            echo "";
            `mysql --user=$MYSQL_POLICY_USERNAME --password="$MYSQL_POLICY_PASSWORD" --host=$MYSQL_SERVER --execute="STATUS" > /dev/null 2> /dev/null`;
            if [ $? != 0 ]; then
                echo "    Could not connect with Policy Service credentials. Will grant Policy Services account privileges.";
                echo -n "- Input the password for the Policy Service account one more time and the user will be added: "
                stty -echo;
                read MYSQL_POLICY_PASSWORD2;
                stty echo;
                if [ "$MYSQL_POLICY_PASSWORD2" == "$MYSQL_POLICY_PASSWORD" ]; then
                    MYSQL_GOT_POLICY_CREDENTIALS=1;
                    echo "    Passwords match.";
                else
                    echo "    Passwords do not match. Starting over.";
                fi
                MYSQL_POLICY_HAS_ACCESS=0;
            else
                echo "    Policy Service account access verified.";
                MYSQL_POLICY_HAS_ACCESS=1;
                MYSQL_GOT_POLICY_CREDENTIALS=1;
            fi
        done
        echo "";
        
        MYSQL_POLICY_DBNAME="oscars_policy";
        if [ -n "$OSCARS_HOME" ] && [ ! -e "$OSCARS_HOME/conf/properties/oscars.properties" ]; then
            echo "hibernate.connection.username=$MYSQL_POLICY_USERNAME" > "$OSCARS_HOME/conf/properties/oscars.properties";
            echo "hibernate.connection.password=$MYSQL_POLICY_PASSWORD" >> "$OSCARS_HOME/conf/properties/oscars.properties";
        elif [ -z "$OSCARS_HOME" ] && [ -n "$CATALINA_HOME" ] && [ ! -e "$CATALINA_HOME/shared/classes/server/oscars.properties" ]; then
            echo "hibernate.connection.username=$MYSQL_POLICY_USERNAME" > "$CATALINA_HOME/shared/classes/server/oscars.properties";
            echo "hibernate.connection.password=$MYSQL_POLICY_PASSWORD" >> "$CATALINA_HOME/shared/classes/server/oscars.properties";
        elif [ -z "$CATALINA_HOME" ] && [ -z "$OSCARS_HOME" ] && [ ! -e "./conf/properties/oscars.properties" ]; then
            echo "hibernate.connection.username=$MYSQL_POLICY_USERNAME" > "./conf/properties/oscars.properties";
            echo "hibernate.connection.password=$MYSQL_POLICY_PASSWORD" >> "./conf/properties/oscars.properties";
        fi

        echo -n "- Got all information. Press return to create the databases...";
        read THROWAWAY;

        echo "    Creating databases $MYSQL_POLICY_DBNAME";
        `mysql --user=$MYSQL_PRIV_USERNAME --password="$MYSQL_PRIV_PASSWORD" --host=$MYSQL_SERVER --execute="CREATE DATABASE $MYSQL_POLICY_DBNAME" > /dev/null 2> /dev/null`;
        echo "    Initializing databases...";
        `mysql --user=$MYSQL_PRIV_USERNAME --password="$MYSQL_PRIV_PASSWORD" --host=$MYSQL_SERVER -D$MYSQL_POLICY_DBNAME < sql/oscars_policy/createTables.sql`;
        echo "    Databases initialized...";
        echo "    Granting privileges to Policy Service account...";
        if [ "$MYSQL_SERVER" == "localhost" ]; then
            MYSQL_POLICY_HOSTNAME="localhost";
        else
            MYSQL_POLICY_HOSTNAME=POLICY_HOSTNAME;
        fi
        `mysql --user=$MYSQL_PRIV_USERNAME --password="$MYSQL_PRIV_PASSWORD" --host=$MYSQL_SERVER --execute="GRANT INSERT, SELECT, DELETE, UPDATE ON $MYSQL_POLICY_DBNAME.* TO '$MYSQL_POLICY_USERNAME'@'$MYSQL_POLICY_HOSTNAME' IDENTIFIED BY '$MYSQL_POLICY_PASSWORD'"`;
        echo "    Policy Service account authorized.";
        if [ $MYSQL_SERVER == "localhost" ]; then
            MYSQL_SERVER="";
        fi
    fi
fi

echo -n "- Press return to build Policy Service...";
read PRESS_RETURN;

echo "";
echo "";
echo "--- Cleaning before compile...";
ant clean;
echo "";
echo "";
echo "--- Compiling...";
ant compile;


echo "";
echo "";
echo "--- Policy Service built.";

echo "";
echo "##############################################################################";
echo "";
echo "Policy Service built. Please run do_install.sh to complete this installation.";
echo "";
echo "##############################################################################";
