-- Database that stores policy information
CREATE DATABASE IF NOT EXISTS oscars_policy;
USE oscars_policy;

-- Table structure for table lTypes
--
CREATE TABLE IF NOT EXISTS lTypes (
  id int(11) NOT NULL AUTO_INCREMENT,
  name text NOT NULL,
  PRIMARY KEY (id),
  UNIQUE KEY lTypesName (name(20))
) ENGINE=MyISAM;
LOCK TABLES lTypes WRITE;
INSERT INTO lTypes VALUES (1,'XPATH_NODE'),(2,'XPATH_NUMBER'),(3,'XPATH_BOOLEAN'),(4,'XPATH_STRING'),(5,'XPATH_LIST'),(6,'XPATH_ENUM'),(7,'ACTION'),(8,'SPECIAL_VAR'),(9,'RULE'),(10,'*');
UNLOCK TABLES;

--
-- Table structure for table rTypes
--
CREATE TABLE IF NOT EXISTS rTypes (
  id int(11) NOT NULL AUTO_INCREMENT,
  name text NOT NULL,
  PRIMARY KEY (id),
  UNIQUE KEY rTypesName (name(20))
) ENGINE=MyISAM;
LOCK TABLES rTypes WRITE;
INSERT INTO rTypes VALUES (1,'STRING'),(2,'NUMBER'),(3,'BOOLEAN'),(4,'DATE'),(5,'RULE'),(6,'*');
UNLOCK TABLES;
--
-- Table structure for table operators
--
CREATE TABLE IF NOT EXISTS operators (
  id int(11) NOT NULL AUTO_INCREMENT,
  rTypeId int(11) NOT NULL,
  name text NOT NULL,
  PRIMARY KEY (id),
  UNIQUE KEY operatorTypeName (rTypeId,name(20))
) ENGINE=MyISAM;
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="RULE"), "OR");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="RULE"), "AND");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="BOOLEAN"), "==");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="BOOLEAN"), "!=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="STRING"), "==");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="STRING"), "!=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="STRING"), "startsWith");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="STRING"), "endsWith");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="STRING"), "contains");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="NUMBER"), "==");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="NUMBER"), "!=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="NUMBER"), "<");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="NUMBER"), ">");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="NUMBER"), "<=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="NUMBER"), ">=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="DATE"), "==");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="DATE"), "!=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="DATE"), "<");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="DATE"), ">");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="DATE"), "<=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="DATE"), ">=");
INSERT INTO operators VALUES(NULL, (SELECT id FROM rTypes WHERE name="*"), "==");

--
-- Table structure for table policyClassCriteria
--
CREATE TABLE IF NOT EXISTS policyClassCriteria (
  id int(11) NOT NULL AUTO_INCREMENT,
  policyClassId int(11) NOT NULL,
  typeId int(11) NOT NULL,
  value text,
  PRIMARY KEY (id),
  UNIQUE KEY criteriaClassTypeVal (policyClassId,typeId,value(100))
) ENGINE=MyISAM;

--
-- Table structure for table policyClasses
--
CREATE TABLE IF NOT EXISTS policyClasses (
  id int(11) NOT NULL AUTO_INCREMENT,
  parentId int(11) DEFAULT NULL,
  name text NOT NULL,
  ruleSetId int(11) NOT NULL,
  allowOnMatch tinyint(1) NOT NULL DEFAULT '1',
  PRIMARY KEY (id)
) ENGINE=MyISAM;

--
-- Table structure for table policyClassCriteriaTypes
--
CREATE TABLE IF NOT EXISTS policyClassCriterionTypes (
  id int(11) NOT NULL AUTO_INCREMENT,
  name text NOT NULL,
  PRIMARY KEY (id),
  UNIQUE KEY typeName (name(20))
) ENGINE=MyISAM;
LOCK TABLES policyClassCriterionTypes WRITE;
INSERT INTO policyClassCriterionTypes VALUES (1,'NAME'),(2,'ATTR_ROLE');
UNLOCK TABLES;

--
-- Table structure for table rules
--
CREATE TABLE IF NOT EXISTS rules (
  id int(11) NOT NULL AUTO_INCREMENT,
  lTypeId int(11) NOT NULL,
  lValue text NOT NULL,
  operatorId int(11) NOT NULL,
  rValue text,
  PRIMARY KEY (id)
) ENGINE=MyISAM;

--
-- Table structure for table ruleSets
--
CREATE TABLE IF NOT EXISTS ruleSets (
  id int(11) NOT NULL AUTO_INCREMENT,
  name text NOT NULL,
  ruleId int(11) NOT NULL,
  PRIMARY KEY (id),
  UNIQUE KEY ruleSetName (name(20))
) ENGINE=MyISAM;

-- Add default policy class
INSERT INTO rules VALUES(NULL, (SELECT id FROM lTypes WHERE name='*'), '*', (SELECT id FROM operators WHERE name='==' AND rTypeId=(SELECT id FROM rTypes WHERE name='*')), '*');
INSERT INTO ruleSets VALUES(NULL, 'default', (SELECT id FROM rules));
INSERT INTO policyClasses VALUES(NULL, NULL, 'default', 1, (SELECT id FROM ruleSets WHERE name='default'));

