package net.es.oscars.policy.client;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * A generic check policy request. Stores the list of requesters of type S, the action and a list
 * of resources of type T. The S type should always be convertible to a Requester type defined in 
 * the parent package of this class. Requester exist that keep information in OpenSAML types but this 
 * may not be desirable because the OpenSAML types are not Serializable. Other Requester types exist 
 * that store types as Serializable objects, such as DOMRequester which uses W3C DOM.The T type is intended 
 * to be a generic element such as a particular DOM's Element implementation.
 * 
 * @author Andrew Lake (alake@internet2.edu)
 * @param <S> the type of Requester such as Requester or DOMRequester.
 * @param <T> the type of resource. For example, a particular DOM's element.
 */
public class CheckPolicyRequest<S,T> implements Serializable{
    protected List<S> requesters;
    protected String action;
    protected List<T> resources;
    
    public CheckPolicyRequest(){
        this.requesters = new ArrayList<S>();
        this.action = null;
        this.resources = new ArrayList<T>();
    }
    
    /**
     * @return the list of the requesters about whom a policy decision is needed
     */
    public List<S> getRequesters() {
        return this.requesters;
    }
    /**
     * @param requesters the requesters to set
     */
    public void setRequesters(List<S> requesters) {
        this.requesters = requesters;
    }
    /**
     * @return the action the requesters would like to perform
     */
    public String getAction() {
        return this.action;
    }
    
    /**
     * @param action the action to set
     */
    public void setAction(String action) {
        this.action = action;
    }
    /**
     * @return the resources on which the requesters which to perform the given action. 
     */
    public List<T> getResources() {
        return this.resources;
    }
    
    /**
     * @param resources the resources to set
     */
    public void setResources(List<T> resources) {
        this.resources = resources;
    }
    
    
}
