package net.es.oscars.policy.database;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import net.es.oscars.database.HibernateBean;

/**
 * Associates a Requester that matches a set of criteria to a set of rules. 
 * If criteria met and rules match then will be allowed if allowOnMatch is true
 * and denied if not matched (and vice versa if allowsOnMatch is false).
 * 
 * @author Andrew Lake (alake@internet2.edu)
 */
public class PolicyClass extends HibernateBean implements Serializable {
    /** persistent field */
    private String name;
    
    /** persistent field */
    private Boolean allowOnMatch;
    
    /** persistent field */
    private PolicyClass parent;
    
    /** persistent field */
    private RuleSet ruleSet;
    
    /** persistent field */
    private Set<PolicyClassCriterion> criteria = new HashSet<PolicyClassCriterion>();
    
    /**
     * @return the name of the policy class
     */
    public String getName() {
        return this.name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return true if matching request will be allowed. False if matching requests should be denied.
     */
    public Boolean getAllowOnMatch() {
        return this.allowOnMatch;
    }

    /**
     * @param allowOnMatch true if matching request will be allowed. False if matching requests should be denied.
     */
    public void setAllowOnMatch(Boolean allowOnMatch) {
        this.allowOnMatch = allowOnMatch;
    }

    /**
     * @return the parent policy class of this policy class if it exists. It may be NULL if root policy class.
     */
    public PolicyClass getParent() {
        return this.parent;
    }

    /**
     * @param parent the parent policy class of this policy class if it exists
     */
    public void setParent(PolicyClass parent) {
        this.parent = parent;
    }

    /**
     * @return the set of rules associated with this policy
     */
    public RuleSet getRuleSet() {
        return this.ruleSet;
    }

    /**
     * @param ruleSet the set of rules associated with this policy
     */
    public void setRuleSet(RuleSet ruleSet) {
        this.ruleSet = ruleSet;
    }

    /**
     * @return the criteria
     */
    public Set<PolicyClassCriterion> getCriteria() {
        return this.criteria;
    }

    /**
     * @param criteria the criteria to set
     */
    public void setCriteria(Set<PolicyClassCriterion> criteria) {
        this.criteria = criteria;
    }
    
}
